/*
 * Programmer:  Timothy E. Hoff
 * File Name:   mailer.c
 * Written:     July 7, 1996
 * Modified:
 * Description: This module provides mail functionality to sfw.
 *
 * Copyright (c) 1996, Timothy E. Hoff
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <string.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/utsname.h>
#include <time.h>
#include <errno.h>
#include "log.h"
#include "sizes.h"
#include "util.h"
#include "mailer.h"

#define PATH_MAILER "/usr/bin/mail"

#ifndef _LINT
static char rcsid[] = "$Id: mailer.c,v 2.2 1996/07/22 20:42:34 t07de7 Exp $";
#endif
 
void sendmsg(char *address, char *user, char flag, char **argv)
{
    /*
     * Send a mail message to address notifying the admin
     * of activity.  Flag is a '+' is access was granted
     * otherwise it is a '-'.  argv is the command line.
     */

    char *filename;                 /* temporary file */
    static char msgbuf[8192];
    static char binmail[MAXLINE];
    static char option[3];
    static char subject[MAXLINE];
    static char addr[MAX_ADDRESS];
    pid_t pid;
    int fd, status;
        
    if ((pid = fork()) == 0) {      /* first child */
        filename = tmpnam(NULL);

        compose(msgbuf, user, flag, argv);
                
        if (writemsg(filename, msgbuf) == 0) {
            if ((pid = fork()) == 0) {          /* second child */
                if ((fd = open(filename, O_RDONLY)) == -1) {
                    log_error(ENOENT, "Could not open temporary file.");
                    exit(EXIT_FAILURE);
                } else {
                    close(STDIN_FILENO);
                    dup2(fd, STDIN_FILENO);
                }
                
                strncpy(binmail, PATH_MAILER, MAXLINE);
                strcpy(option, "-s");
                sprintf(subject, "%s access notification", strip_path(*argv));
                strncpy(addr, address, MAX_ADDRESS);
    
                execl(binmail, binmail, option, subject, addr);
                log_error(errno, "Could not execute mail client.");
                exit(EXIT_FAILURE);
            } else {                            /* parent */
                wait(&status);
                unlink(filename);
                exit(EXIT_SUCCESS);
            }
        } else {                                /* writemsg failed */
            return;
        }
    } else {
        return;
    }
}


void compose(char *buf, char *user, char flag, char **argv)
{
    /*
     * Compose the mail message and save it to buf.
     */
     
    char cmdline[MAXLINE];
    char *program;              /* program name (argv[0]) */
    char *ttyname;              /* controlling terminal */
    char *hostname;
    struct utsname hostinfo;
    time_t today;
    char *title = "This message was generated by Simple File Wrapper on";

    program = *argv;
    cmdline[0] = '\0';

    if ((ttyname = getctty()) == NULL) {
        ttyname = "no TTY";
    }
    
    if (uname(&hostinfo) == 0) {
        hostname = hostinfo.nodename;
    } else {
        hostname = "UNKNOWN";
    }

    (void) time(&today);
    
    while (*argv != NULL) {
        strncat(cmdline, *argv, (MAXLINE - strlen(cmdline)));
        strncat(cmdline, " ", (MAXLINE - strlen(cmdline)));
        argv++;
    }

    sprintf(buf, "\n%s %s\nUser %s was %s access to %s running on \nhost %s [%s]\
.\nThe command to be executed was:\n\t%s",
        title, ctime(&today), user, (flag == '+' ? "granted" : "denied"),
        program, hostname, ttyname, cmdline);

    return;
}


int writemsg(char *pathname, char *msg)
{
    int fd;
    int bc;
    char errmsg[MAXLINE];
    extern int errno;
    
    if ((fd = open(pathname, O_WRONLY | O_CREAT, S_IRUSR | S_IWUSR)) == -1) {
        sprintf(errmsg, "Could not create temporary file %s.\n", pathname);
        log_error(errno, errmsg);
        return -1;
    }

    bc = strlen(msg);

    if (write(fd, msg, bc) != bc) {
        close(fd);
        return -1;
    } else {
        close(fd);
        return 0;
    }   
}
